#ifndef UNITS_H
#define UNITS_H

#ifdef REDUNITS
/* Reduced units - all factors=1. Not tested recently!!! */
#define massunit 1
#define lengthunit 1
#define timeunit 1
#define munit 1
#define Eunit 1
#define Punit 1
#define rhounit 1

#define kcal 1
#define Debye 1
#define electron 1

#else
/* real units */

/*
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                       units of measurement                      %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MACSIMUS may records the results either in the internal program units or
SI units

In many cases the user output is in SI units, which is indicated in
[brackets].  If no units are given, the internal program units are used.

WARNING: The author of this program thinks that anybody using non-SI units
should be fined and if does not pay then hanged by legs in a chilly draft.
Unfortunately, direct using of SI units would cause over- and underflows in
calculations so that special units had to be used internally in the program.
In addition, some multiplications are spared if the unit of charge is chosen
so that the electrostatic energy is charge*charge/length

Values based on CODATA 2014 (http://physics.nist.gov/cuu/Constants/index.html)

===============================================================================
quantity    | program unit |value in SI and other units
------------+--------------+---------------------------------------------------
temperature |K             |1 K
length      |A             |Angstroem = 1e-10 m
time        |ps            |1e-12 s
energy      |k*K           |1.38064852e-23 J = 8.3144598 J/mol= 1.9872036 cal/mol
            |              |
velocity    |A/ps          |100 m/s
mass        |k*K/(A/ps)^2  |1.38064852e-27 kg = 8.3144598 g/mol
mass density|k*K*ps^2/A^5  |1380.64852 kg/m^3
pressure    |k*K/A^3       |13806485.2 Pa
charge      |(k*K*A)^1/2   |SI: (k*K*A*1e7*F/m)^1/2/{c} = 3.91941150e-22 C
            |              |where c={c} m/s, {c}=299792458, and F=Farad=C^2/J
            |              |CGS: 1.175010008e-12 g^1/2 cm^3/2 s^-1
dipole      |A*(k*K*A)^1/2 |SI: 3.91941150e-32 C m
moment      |              |CGS: 1.175010008e-20 g^1/2 cm^5/2 s^-1
            |              |     = 0.01175010008 Debye
polariz-    |A^3           |SI: 10e-23 {c}^-2 C m^2/V = 1.112650056e-40 C m^2/V
ability     |              |CGS: 10e-24 cm^3
===============================================================================

NOTES:
  k = Boltzmann constant = 1.38064852(79)e-23 J/K
  (loosely, energy is measured in Kelvins)
  A = Angstrom
  1 cal = thermochemical calorie = 4.184 J
  program electrostatic energy is q1*q2/r
  charge of electron is 408.7798945 charge units, e^2=167101.002 charge units^2
  1 mol = 6.022140857e23 (74) (Avogadro number)
  SI polarizability volume = SI polarizability/(4 PI epsilon_0)
  The values are consistent within error bars
  1/(4 PI eps_0) = {c}^2 1e-7 = 8987551787 [Jm/C^2=m/F]
  1 e = 1.6021766208e-19 (98) C = 96485.3329 C/mol (Faraday constant)
  e^2/(4 PI eps_0)/A = 2.30707751e-18 J = 1389354.58 J/mol 
    = 332.063713 kcal/mol (directly) 
    = 332.063710541 kcal/mol (rounding-compatible with electron^2 below)
  1 e A = 4.80320467 Debye
  1 C m = 2.99792458e+29 Debye
(See also evu and .evudata)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

/*** basic SI factors ***/
#define massunit 1.38064852e-27 /* kg */
#define lengthunit 1e-10       /* m */
#define timeunit   1e-12       /* s */
#define chargeunit 3.91941150e-22 /* C (some quantities may contain 4 pi) */
#define energyunit 1.38064852e-23 /* J */
#define Avogadro (1e-3*munit/massunit) /* 1/mol */
/* Examples:
   1) viscosity has dimensionality  mass length^-1 time^-1 = pressure*time
      the program unit of viscosity is thus
        viscunit=massunit/lengthunit/timeunit
                =Punit*timeunit
                =1.38064852e-05 Pa.s
        viscosity[SI] = viscunit * programviscosity
   2) electric current density has dimensionality  charge length^-2 time^-1
      the program unit of current density is thus
        Junit=chargeunit/lengthunit^2/timeunit
        3.91941150e+10 A/m2
      J[SI] = Junit * programJ
*/

/*** some derived program units in SI ***/
#define munit 0.83144598    /* g/mol */
#define Eunit 8.3144598     /* J/mol */
#define Punit 13806485.2    /* Pa */
#define rhounit 1380.64852  /* kg/m^3 */

#define Debye 0.01175010008 /* this is 1 Debye in program units */
#define kcal 4184.0         /* J (some criminals still do use calories) */

#define electron 408.7798945 /* charge of electron in program units */


#endif
#endif /* UNITS_H */
